/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package lufti.fractrees;

import java.util.ArrayList;

/**
 *
 * @author reidl
 */
public abstract class Turtle {

	protected TurtleState currentState;
	private final double delta, len;
	private final ArrayList<TurtleState> stack;

	public Turtle(double angle, double delta, double len) {
		this.delta = delta;
		this.len = len;

		currentState = new TurtleState(0, 0, angle);
		stack = new ArrayList<TurtleState>();
	}

	public void push() {
		stack.add(currentState.copy());
	}

	public void pop() throws TurtleException {
		if (stack.isEmpty()) {
			throw new TurtleException.EmptyStackException(this);
		}
		currentState = stack.remove(stack.size() - 1);
	}

	public abstract void stepAndDraw() throws TurtleException;

	public void step() {
		currentState.x += Math.cos(currentState.angle) * len;
		currentState.y += Math.sin(currentState.angle) * len;
	}

	public void turnLeft() {
		currentState.angle -= delta;
		while (currentState.angle < 2 * Math.PI) {
			currentState.angle += 2 * Math.PI;
		}
	}

	public void turnRight() {
		currentState.angle += delta;
		while (currentState.angle >= 2 * Math.PI) {
			currentState.angle -= 2 * Math.PI;
		}
	}

	public final void execute(String command) throws TurtleException {
		int len = command.length();
		for (int i = 0; i < len; i++) {
			char cmd = command.charAt(i);
			if (cmd == 'F') {
				stepAndDraw();
			} else if (cmd == 'f') {
				step();
			} else if (cmd == '+') {
				turnRight();
			} else if (cmd == '-') {
				turnLeft();
			} else if (cmd == '[') {
				push();
			} else if (cmd == ']') {
				pop();
			} else {
				throw new TurtleException.InvalidSymbolException(this,cmd);
			}
		}
	}

	protected static class TurtleState {
		protected double x, y, angle;

		public TurtleState(double x, double y, double angle) {
			this.x = x;
			this.y = y;
			this.angle = angle;
		}

		public TurtleState copy() {
			return new TurtleState(x, y, angle);
		}
	}
}
